#ifdef __USAGE
%C - fax class 2 modem handler (QNX)

%C	[options]                        (receive a fax)
%C	-S number [options] file ...     (send a fax)
Options:
 -D file|con   Debug mode.
 -m modem      Modem device to use. (default: standard input/output)
 -r            Reverse bit order of data to the modem.
 -L directory  Log files and received faxes. (default: /usr/spool/fax)
 -d            Reset mode back to data mode when done.

Send only options:
 -F fax_id     Local fax machine identification. (default: "Unknown")
 -i string     Initalize modem. (default: "ATZ")
 -l resolution Verticle resolution 0=98, 1=198. (default: 1)
 -S [P]number  Dial this number, then send a fax.
 -x            Support xon/xoff flow control as well as rts/cts (buggy modems).

Receive only options:
 -a            Answer calls directly. (default: assume modem utility answers)
 -b baud       Set baud rate and rts/cts flow control. (default: 19200)
#endif

/*
 This program handles Class 2 FAX modems only. It does not support class 1.
 It requires that your modem operate with hardware flow control (CTS/RTS)
 at a fixed baud rate. The baud rate should be established using the modem
 or stty utility. For answering calls you may use the modem utility which
 execs into this fax utility (default) or you may have the fax utility answer
 the call itself (-a option). If fax answers the call then it will typically
 be started by the tinit utility. For sending faxes this utility will typically
 be invoked be a system spooler.
*/

#include <unistd.h>
#include <time.h>
#include <termios.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <signal.h>
#include <setjmp.h>
#include <process.h>
#include <fcntl.h>
#include <env.h>
#include <ctype.h>
#include <stddef.h>

#include <sys/qfx.h>

#include <sys/stat.h>
#include <sys/dev.h>
#include <sys/qioctl.h>
#include <sys/timers.h>
#include <sys/proxy.h>
#include <sys/qnx_glob.h>
#include <sys/kernel.h>

// General Definitions
#define	TRUE	1
#define	FALSE	0

#define ETX		0x03
#define DLE		0x10
#define DC2		0x12
#define SUB		0x1a
#define CAN		0x18

#define FCON		1
#define FDCS		2
#define FDIS		3
#define FCFR		4
#define FTSI		5
#define FCSI		6
#define FPTS		7
#define FET 		8
#define FHNG		9
#define CONNECT		10
#define OK			11
#define RING		12
#define BUSY		13
#define NOCARRIER	14
#define NODIALTONE	15
#define NOANSWER	16

typedef struct keyword_entry {
	char	*string;
	int		 token;
	} KEYWORD;

struct qfx_header header = { { "qfx" } };


// Exit status codes
// Note that codes >= 10 usually indicate that something must be done before
// retrying
//
#define FAX_OK			0	// All pages sent/received successfully
#define FAX_BUSY		1	// The called number was busy
#define FAX_DEV_INUSE	2	// The fax modem device is in use
#define FAX_NOCARRIER	3	// No carrier
#define FAX_TIMEOUT		4	// A timeout waiting for an expected modem response
#define FAX_MWRITE		5	// Error writing to modem
#define FAX_CMDLINE		10	// An error processing the command line
#define FAX_DOPEN		11	// Unable to open a fax data file
#define FAX_LOPEN		12	// Unable to open a fax log file
#define FAX_MOPEN		13	// Unable to open fax modem device
#define FAX_SIGTERM		14	// Shutdown due to a received signal
#define FAX_NOFILES		15	// No files were specified for a send
#define FAX_FREAD		16	// Error reading from file
#define FAX_FWRITE		17	// Error writing to file
#define FAX_HANGUP		18	// A hangup with a error code was returned


// Prototypes
int				execute(int (*func)(), unsigned);
int				get_modem(char *, unsigned, unsigned);
void			put_modem(char *);
long			number(char *, char *);
void			debugmsg(const char *, ...);
void			sleepms(long msec);
int				rdata(void);
int				sdata(void);
void			sigterm(int);
void			shutdown(int);
void			open_debug_file(void);
void			chk_header(FILE *);
int				faxcall(int);
int				faxsnd(int);
int				faxanswer(int);
int				faxrcv(int);
unsigned char	swapbits(unsigned char);
char			*get_farg(int);
void			set_line(speed_t);


// Globals
char	 dle = DLE;
char	 etx = ETX;
char	 dc2 = DC2;
char	 can = CAN;
int		 phase;
int		 answer;
int		 qfx;
int		 xonxoff;
int		 newres = 1;
int		 resolution = 1;
int		 page = 1;
int		 nfiles = 1;			// Number of files to send
int		 filen;					// Current file being sent (0 .. nfiles-1)
int		 datamode;				// Reset modem to data mode when done
int		 verbose;
char	*(*filev)[];			// List of files to send
int		 sending;				// Non-zero if we are sending a fax
int		 reverse;
int		 in_data_mode;
int		 digits;				// Number of digits in the dialout string.
char	 first;					// The first digit in the string.
time_t	 startsec;
speed_t	 baud = 19200;
char	 fbuf[100];
char	 fpbuf[100];
char	 faxfname[100];
char	*mdmname;
char	*dbgfname;
char	*faxdname = "/usr/spool/fax";
char	*hayes_init ="ATZ";
char	atd[80];				// An optional number to dial on a send
char	*faxid = "Unknown";		// Fax id to xmit to remote fax machine
FILE	*debug_fp;				// Send debug info out here
FILE	*sfile_fp;				// Input file for sent FAX data
FILE	*rfile_fp;				// Output file for received FAX data
int		 termio_valid;
struct termios	termio;
struct stat		sbuf;

KEYWORD keywords[] ={
	{"+FCON",		FCON},
	{"+FDCS",		FDCS},
	{"+FDIS",		FDIS},
	{"+FCFR",		FCFR},
	{"+FTSI",		FTSI},
	{"+FCSI",		FCSI},
	{"+FPTS",		FPTS},
	{"+FET",		FET },
	{"+FHNG",		FHNG},
	{"CONNECT",		CONNECT},
	{"OK",			OK},
	{"RING",		RING},
	{"BUSY",		BUSY},
	{"NO CARRIER",	NOCARRIER},
	{"NO DIALTONE",	NODIALTONE},
	{"NO ANSWER",	NOANSWER},
	NULL
	} ;


main( argc, argv )
	int		argc;
	char	*argv[];
	{
	int		opt, status, signo;
	char	*cp;

	startsec = time(NULL);

	while((opt = getopt(argc, argv, "ab:dD:F:i:l:L:m:rS:vx")) != -1) {
		switch(opt) {
		case 'a':
			answer = 1;
			break;

		case 'b':
			baud = number(optarg, "-b");
			break;

		case 'd':
			datamode = 1;
			break;

		case 'D':
			dbgfname = optarg;
			break;

		case 'F':
			faxid = optarg;
			break;

		case 'i':
			hayes_init = optarg;
			break;

		case 'l':
			resolution = number(optarg, "-l");
			break;

		case 'm':
			close(0);
			close(1);
			if(open(mdmname = optarg, O_RDWR) != 0)
				shutdown(FAX_MOPEN);
			if(dup2(0, 1) == -1)
				shutdown(FAX_MOPEN);
			break;

		case 'r':
			reverse = 1;
			break;

		case 'S':
			for(cp = optarg, digits = 0 ; *cp ; ++cp)
				if(*cp >= '0'  &&  *cp <= '9')
					if(++digits == 1)
						first = *cp;

			// Be nice and handle case where leading '1' is missing for
			// a north american number.
			if(digits == 10  &&  first > '1')
				sprintf(atd, "ATDT1%s", optarg);
			else
				sprintf(atd, "ATDT%s", optarg);
			sending = 1;
			break;

		case 'v':
			verbose = 1;
			break;

		case 'x':
			xonxoff = 1;
			break;

		default:
			shutdown(FAX_CMDLINE);
			}
		}

	dev_mode(0, 0, _DEV_MODES);			/* Go raw						*/
	set_line(baud);
	for(signo = _SIGMIN ; signo <= _SIGMAX ; ++signo)
		signal(signo, &sigterm);
	signal(SIGHUP, SIG_IGN);

	if(sending) {
		struct _dev_info_entry devinfo;

		if((nfiles = argc - optind) == 0)
			shutdown(FAX_NOFILES);
		
		filev = &argv[optind];
		strcpy(faxfname, (*filev)[0]);
		open_debug_file();

		dev_info(0, &devinfo);
		if(mdmname  &&  devinfo.open_count > 1)
			shutdown(FAX_DEV_INUSE);

		sfile_fp = fopen(faxfname, "r");
		if(sfile_fp == NULL)
			shutdown(FAX_DOPEN);
		fstat(fileno(sfile_fp), &sbuf);
		chk_header(sfile_fp);

		if(strlen(atd) > 5) {
			put_modem(hayes_init);
			sprintf(fbuf, "AT+FCLASS=2|AT +FCR=1 +FLID=\"%-20.20s\"", faxid);
			put_modem(fbuf);
			if(digits > 8)
				put_modem("ATS7=120");
			put_modem(atd);
			if(status = execute(&faxcall, 600))
				shutdown(status);
			}

		phase = 'B';
		status = execute(&faxsnd, 300);
		}
	else {
		sprintf(faxfname, "%s/%10.10lu.qfx", faxdname, time(NULL));
		open_debug_file();

		if(answer) {
			put_modem(hayes_init);
			sprintf(fbuf, "AT+FCLASS=2|AT+FAA=0 +FCR=1 +FLID=\"%-20.20s\"", faxid);
			put_modem(fbuf);

			if(status = execute(&faxanswer, 6000))
				shutdown(status);
			}

		if((rfile_fp = fopen(faxfname, "w")) == NULL)
			shutdown(FAX_DOPEN);

		status = execute(&faxrcv, 300);
		}

	shutdown(status);
	}


//
// Execute a state machine to interact with the modem.
// Returns:
//   0  Success
//  !0  An error code 1..127
//
execute(int (*func)(), unsigned timeout) {
	unsigned n;
	int status;
	KEYWORD *kp;

	for(;;) {
		n = get_modem(fbuf, sizeof(fbuf), timeout);	// Read modem msg
		if(n == 0)
			return(FAX_TIMEOUT);

		for(kp = &keywords[0] ; kp->string ; ++kp) {
			if(strncmp(fbuf, kp->string, n = strlen(kp->string)) == 0) {
				debugmsg("Match \"%s\"\n", fbuf);
				if((status = (*func)(kp->token)) & 0x80)
					return(status & 0x7f); 
				break;
				}
			}
		}
	}


/*
 Get a pointer to the start of an argument from a command response
 from the modem. Responses with arguments are of the form
        1   2   3  (values of argno)
 +Fxxx:arg,arg,arg ...
 If the requested argument is not present then a pointer to a null
 string is returned. Note that the returned argument pointer is not
 itself null terminated. The only null will be at the end of all the
 arguments. This does not matter since atoi() will be used to parse
 numeric values and it stops on the first nondigit (like a comma).
*/

char *
get_farg(int argno) {
	char *str = fbuf;

	// Skip over to ':'
	while(*str  &&  *str != ':') ++str;

	while(argno) {
		// Skip leading spaces
		while(*str  &&  *++str == ' ') ;
		if(*str == 0  ||  --argno == 0)
			break;

		// Skip current argument
		while(*str  &&  *str != ',') ++str;
		}

	return(str);
	}



faxcall(int token) {

	switch(token) {
	case OK:
		return(0);

	case FCON:
		return(0x80 | 0);

	case BUSY:
		return(0x80 | FAX_BUSY);

	case FHNG:
	case NOCARRIER:
	case NOANSWER:
	case NODIALTONE:
		return(0x80 | FAX_NOCARRIER);
		}

	return(0);
	}


faxsnd(int token) {
	static int status;
	char buf[20];

	switch(token) {
	case FDIS:
		return(0);

	case OK:
		if(phase == 'B') {
			if(newres) {
				newres = 0;
				sprintf(buf, "AT+FDIS=%d", header.resolution);
				put_modem(buf);
				}
			if(filen < nfiles) {
				put_modem("AT+FDT");
				in_data_mode = 1;
				phase = 'C';
				}
			}
		else {
			if(status == 0  ||  nfiles - filen > 1)
				put_modem("AT+FET=0");
			else
				put_modem("AT+FET=2");
			phase = 'B';
			}
		return(0);

	case FDCS:
		return(0);

	case CONNECT:
		status = sdata();
		in_data_mode = 0;
		return(0);

	case FPTS:
		if(status) {
			if(++filen < nfiles) {
				strcpy(faxfname, (*filev)[filen]);
				sfile_fp = fopen(faxfname, "r");
				if(sfile_fp == NULL) {
					shutdown(FAX_DOPEN);
					}
				chk_header(sfile_fp);
				}
			}
		return(0);

	case FHNG:
		return(0x80 | (atoi(get_farg(1)) ? FAX_HANGUP : 0));
		}

	return(0);
	}


faxanswer(int token) {

	switch(token) {
	case RING:
		put_modem("ATA");
		return(0);

	case OK:
		return(0);

	case FCON:
		return(0x80);

	case FHNG:
	case NOCARRIER:
	case NOANSWER:
	case NODIALTONE:
		return(0x80 | FAX_NOCARRIER);
		}

	return(0);
	}


faxrcv(int token) {
	static int hdr = 0;
	char *cp;
	char buf[100];

	switch(token) {
	case FTSI:
		sprintf(buf, "%10ld FA %s %.30s\n",
				time(NULL), mdmname ? mdmname : ctermid(NULL),
				(cp = strchr(fbuf, '"')) ? cp : "+Unknown");
		logmsg("/etc/acclog", buf);
		strncpy(header.remote_faxid, cp+1, 20);
		return(0);

	case OK:
		put_modem("AT+FDR");
		return(0);

	case FDCS:
		header.resolution = atoi(get_farg(1));
		return(0);

	case FCFR:
		return(0);

	case FPTS:
		return(0);

	case FET:
		return(0);

	case CONNECT:
		if(hdr == 0) {
			hdr = 1;
			if(fwrite(&header, sizeof(header), 1, rfile_fp) != 1)
				shutdown(FAX_FWRITE);
			}
		delay(50);
		tcflush(0, TCIFLUSH);
		write(1, &dc2, 1);
		in_data_mode = 1;
		rdata();
		in_data_mode = 0;
		return(0);

	case FHNG:
		if(rfile_fp)
			fclose(rfile_fp);
		return(0x80 | (atoi(get_farg(1)) ? FAX_HANGUP : 0));
		}

	return(0);
	}


int
rdata() {
	char buf[512];	// Keep this small so we don't consume too far past DLE ETX!
/* was	char buf[12];	*/
/* rk added obuf and optr */
	char obuf [512];
	int optr;
	int i, n, indle, inetx, tries;
	char c;
	
	debugmsg("Fax data start\n");

	for(tries = indle = inetx = 0; inetx == 0 ;) {
		n = dev_read(0, buf, sizeof(buf)-2, sizeof(buf)-2, 5, 0, 0, 0);
/* was	n = dev_read(0, buf, sizeof(buf)-2, 1, 200, 0, 0, 0); */
		if(n == 0) {
			if(++tries >= 10) {
				shutdown(FAX_TIMEOUT);
			}
			continue;
		}
optr=0;
		for(tries = i = 0; i < n; ++i) {
			c = buf[i];
			if(reverse) buf[i] = swapbits(c);
obuf [optr++] = buf [i];

			if(c == DLE) {
				/* RK940807 added to */
				if (indle) {			// this is the 2nd dle...
					optr--;
				}
				/* ... here */
				indle = !indle;
				continue;
			}

			if(indle) {
				indle = 0;
				if(c == SUB) {
					buf[i] = DLE;
					buf[++i] = DLE;		// This is why we read sizeof(buf)-2
					buf[++i] = DLE;
				} else if(c == ETX) {
					++i;
					inetx = 1;
					break;
				} else {
					debugmsg("Unexpected character 0x%2.2x following a DLE.\n", c);
				}
			}
		}

/* was  if(i  &&  fwrite(buf, i, 1, rfile_fp) != 1) { */
		if(optr  &&  fwrite(obuf, optr, 1, rfile_fp) != 1) {
			debugmsg("Error writing file %d\n", errno);
			shutdown(FAX_FWRITE);
		}
	}
	
	fflush(rfile_fp);
	debugmsg("Fax data stop\n");

	return(0);
}


#if 0
int
rdata() {
	char buf[256];
	int base, i, n, indle = 0;
	
	debugmsg("Fax data start\n");

	for(;;) {
		n = dev_read(0, buf, sizeof(buf), 1, 200, 0, 0, 0);
		i = base = 0;
//debugmsg("n=%d\n", n);
		for(; i < n ; ++i) {
			if(buf[i] == DLE) {
				if(indle == 0) {
					indle = 1;
					if(i-base  &&  fwrite(buf+base, i-base, 1, rfile_fp) != 1) {
						debugmsg("Error writing file %d\n", errno);
						shutdown(FAX_FWRITE);
						}
					base = i + 1;
					continue;
					}
				}

			if(indle) {
				indle = 0;
				if(buf[i] == DLE) {
					putc(reverse ? swapbits(DLE) : DLE, rfile_fp);
					}
				else if(buf[i] == ETX) {
					fflush(rfile_fp);
					debugmsg("Fax data stop\n");
					return(0);
					}

				base = i + 1;
				continue;
				}

			if(reverse) buf[i] = swapbits(buf[i]);
			}

		if(i-base  &&  fwrite(buf+base, i-base, 1, rfile_fp) != 1) {
			debugmsg("Error writing file %d\n", errno);
			shutdown(FAX_FWRITE);
			}
		}
	}
#endif


/*
 On return:
	0 - internal page break (qfx format only)
	1 - EOF
	2 - read error
*/
int
sdata() {
	char buf[256];
	unsigned c, i, indle, inetx, status, loop;
	
	debugmsg("Fax data start\n");

	for(i = status = indle = loop = 0;;) {
		inetx = 0;
		if(++loop == 1000) {
			loop = 0;
			while(dev_read(0, &c, 1, 0, 0, 0, 0, 0) == 1) {
				if(verbose) debugmsg("Got %2.2x\n", c);
				if(c == CAN)
					shutdown(FAX_HANGUP);
				}
			}

		if((c = getc(sfile_fp)) == EOF) {
			if(ferror(sfile_fp)) ++status;
			fclose(sfile_fp);
			++status;
			break;
			}

		if(i >= sizeof(buf) - 2) {
			write(1, buf, i);
			i = 0;
			}

		if(reverse)
			c = swapbits(c);
		buf[i++] = c;

		if(c == DLE) {
			indle = !indle;
			if(!qfx)
				buf[i++] = DLE;
			continue;
			}

		if(indle) {
			indle = 0;
			if(c == ETX  &&  qfx) {
				inetx = 1;
				break;
				}
			}
		}

	if(i) {
		write(1, buf, i);
		}

	if(qfx  &&  indle)		// Handles error case where a file ends in DLE only
		write(1, &etx, 1);
	else if(!inetx) {
		write(1, &dle, 1);
		write(1, &etx, 1);
		}
	
	if(status == 0)			// Check if we are at the end of file
		if((c = getc(sfile_fp)) == EOF)
			status = 1;
		else
			ungetc(c, sfile_fp);

	debugmsg("Fax data stop\n");

	return(status);
	}


int
get_modem(char *buf, unsigned len, unsigned timeout)
	{
	char *s = buf;
	int n;

	for(;;) {
		n = dev_read(0, s, 1, 1, 0, timeout, 0, 0);
		if(n == 0)
			return(0);

		if(*s == 0x0d) {			// Check for CR
			if(s == buf)			// Ignore empty lines
				continue;
			break;
			}
		if(*s < ' ')				// Ignore control characters
			continue;
		*s = toupper(*s);			// Convert to a common lower case

		if(s < &buf[len])
			++s;
		}

	*s = '\0';						// Terminate buffer

	debugmsg("Input \"%s\"\n", buf);

	return(strlen(buf));			// Return characters collected
	}



void
put_modem(char *str)
	{
	char c;
	int stop = FALSE;

	if(*str == '\0')
		return;

	debugmsg("Emit \"%s\"\n", str);

	while(!stop) {
		switch(c = *str++) {
			case '\0':	stop = TRUE;				// fall through 
			case '|':	c = '\r';					break;
			case '\\':	c = *str++;					break;
			case '~':	sleep(1);					continue;
			case '^':	tcdropline(0, 1000);		continue;
			case '!':	tcsendbreak(0, 500);		continue;
			case '-':	delay(100);					continue;
			}

		write(1, &c, 1);
		if(c != 0x0d) {				// Attempt to read echo of character
			dev_read(0, &c, 1, 1, 0, 3, 0, 0);	// Wait up to .3 sec
			tcflush(0, TCIFLUSH);	// Flush all input
			}
		else {
			sleep(1);				// Pause 1 second before looping
			if(!stop  &&  *str)		// Flush input if more commands
				tcflush(0, TCIFLUSH);
			}
		}
	}



void
set_line(baud)
speed_t baud;
	{
	struct	 termios newtermio;

	tcgetattr(0, &termio);
	termio_valid = 1;

	newtermio = termio;
	newtermio.c_cflag &= ~(PARENB|PARODD|PARSTK|CSIZE|CSTOPB);
	newtermio.c_cflag |= CS8;

	newtermio.c_cflag |= (IHFLOW|OHFLOW);
	newtermio.c_lflag |= IEXTEN;
	if(xonxoff) {
		newtermio.c_iflag |= IXON;
		newtermio.c_iflag &= ~IXOFF;
		}

	if(baud) {
		cfsetispeed(&newtermio, baud);
		cfsetospeed(&newtermio, baud);
		}

	tcsetattr(0, TCSANOW, &newtermio);
	}



void
chk_header(FILE *fp) {

	newres = 1;
	if(fread(&header, sizeof(header), 1, fp) == 1  &&
	   strcmp(header.magic, "qfx") == 0) {
		qfx = 1;
		if(digits == 0) {
			strcat(atd, header.remote_faxid);
			digits = strlen(header.remote_faxid);
			}
		}
	else {
		fseek(fp, 0L, SEEK_SET);
		qfx = 0;
		header.resolution = resolution;
		}
	}


void
open_debug_file() {
	char buf[100];

	if(dbgfname) {
		if(strcmp(dbgfname, "-") == 0)
			debug_fp = stderr;
		else if(strcmp(dbgfname, "@") == 0) {
			sprintf(buf, "%sd", faxfname);
			debug_fp = fopen(buf, "w");
			}
		else
			debug_fp = fopen(dbgfname, "w");
		}
	}



//
//  Reverses the bit order of a byte.
//
unsigned char
swapbits(unsigned char c) {
    static unsigned char swaptable[256];
    static int i = 0;

	if(i == 0)
	    for (i = 0; i < 256; ++i)
			swaptable[i] =
				((i & 0x01) << 7) |
				((i & 0x02) << 5) |
				((i & 0x04) << 3) |
				((i & 0x08) << 1) |
				((i & 0x10) >> 1) |
				((i & 0x20) >> 3) |
				((i & 0x40) >> 5) |
				((i & 0x80) >> 7);

    return(swaptable[c]);
	}


void
shutdown(int status) {
	FILE *fp;
	char buf[100], tbuf[30];
	time_t sec;
	int fd;
	short int err = status;

	sec = time(NULL);
	strftime(tbuf, sizeof(tbuf), "%y%m%d %H:%M", localtime(&sec));
	sec -= startsec; 

	if(in_data_mode)
		if(sending) {
			write(1, &dle, 1);
			write(1, &etx, 1);
			}
		else {
			write(1, &can, 1);
			}
	else if(status == FAX_TIMEOUT) {
		write(1, &dle, 1);
		write(1, &etx, 1);
		}

	if(datamode) {
		sleep(5);
		put_modem("AT+FCLASS=0");
		}

	if(sending) {
		sprintf(buf, "%s/send.log", faxdname);
		if(fp = fopen(buf, "a")) {
			fprintf(fp, "%s %-16s %2.2d %2.2d %2.2d:%2.2d %5d %s\n",
					tbuf, atd+4, status, header.npages,
					(int) (sec/60), (int) (sec%60),
					sbuf.st_uid,
					*header.remote_contact ? header.remote_contact : faxfname);
			}
		}
	else {
		sprintf(buf, "%s/receive.log", faxdname);
		if(fp = fopen(buf, "a")) {
			fprintf(fp, "%s %-20s %-3d %s\n",
					tbuf, faxid, status, faxfname);
			}
		}

	if(qfx  &&  (fd = open(faxfname, O_RDWR)) != -1) {
		lseek(fd, offsetof(struct qfx_header, err), SEEK_SET);
		write(fd, &err, sizeof(err));
		close(fd);
		}

	if(termio_valid)
		tcsetattr(0, TCSANOW, &termio);

	exit(status ? (status < 10 ? 1 : 2) : 0);
	}


void
sigterm(int signo) {

	debugmsg("Shutdown via signal %d.\n", signo);
	shutdown(FAX_SIGTERM);
	}


void
sleepms(long msec)
	{
	timer_t tid;
	struct itimerspec timer;

	tid = mktimer(TIMEOFDAY, _TNOTIFY_SLEEP, 0);
	timer.it_value.tv_sec     = msec/1000L;
	timer.it_value.tv_nsec    = (msec%1000L) * 1000000L;
	timer.it_interval.tv_sec  = 0L;
	timer.it_interval.tv_nsec = 0L;

	reltimer(tid, &timer, NULL);
	}


long
number(str, opt)
char *str, *opt;
	{
	long n;
	char *endptr;

	opt = opt;
	n = strtol(str, &endptr, 10);

	if(*endptr != '\0')
		shutdown(FAX_CMDLINE);

	return(n);
	}


void
logmsg(file, str)
char *file, *str;
	{
	int fd;

	if((fd = open(file, O_WRONLY|O_APPEND)) != -1) {
		write(fd, str, strlen(str));
		close(fd);
		}
	}


void
debugmsg(const char *format, ...) {
	va_list arglist;

	if(debug_fp == NULL)
		return;

	fprintf(debug_fp, "fax %s ", mdmname ? mdmname : ctermid(NULL));

	va_start(arglist, format);
	vfprintf(debug_fp, format, arglist);

	fflush(debug_fp);
	}
